
#include "forward.h"
#include "global.h"
#include "toolbox.h"
#include "main.h"
#include "vjbox.h"


extern VJBox *vjbox;


#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <string.h>
#include <time.h>

#ifdef HAVE_BROKEN_AX25_H
#include "../linkt/local_ax25.h"
#include <netrose/rose.h>
#elif HAVE_NETAX25_AX25_H
#include <netax25/ax25.h>
#include <netrose/rose.h>
#else
#include <linux/ax25.h>
#include <linux/rose.h>
#endif


#ifdef HAVE_NETAX25_AXLIB_H
   #include <netax25/axconfig.h>
   #include <netax25/axlib.h>
#elif HAVE_AX25_AXUTILS_H
   #include <ax25/axconfig.h>
   #include <ax25/axutils.h>
#else
   #include <axconfig.h>
   #include <axutils.h>
#endif


#define STATUS_DEFAULT 0
#define STATUS_SENDSID 1
#define STATUS_GETSENDANSW 2
#define STATUS_WAITSENDMSG 3
#define STATUS_WAITSENDCMD 4
#define STATUS_GETSUBJECT 7
#define STATUS_GETMAIL 6



Forward::Forward()
{
	savestr = NULL;
	savestrlen = 0;
	status = STATUS_DEFAULT;
   linemode = true;
   sendstr = NULL;
   sendstrlen = 0;
   filefd = -1;

	AX25connect();
}


Forward::~Forward()
{
}


void Forward::AX25connect()
{
	struct full_sockaddr_ax25 ax25;
	int addrlen;
	struct full_sockaddr_ax25 ax25_d;
	int addrlen_d, i;
	char tmp[500];



	if (ax25_config_load_ports() == 0)
	{
   	printf("Error: No AX.25 port data available\n\n");
      exit(1);
	}

	if (ax25_config_get_addr(config->ax25port) == NULL)
	{
      printf("Error: Invalid port setting for port %s.\n\n", config->ax25port);
      exit(1);
	}

	paclen = ax25_config_get_paclen(config->ax25port);




	if (config->ax25path == NULL)
		strcpy(tmp, config->ax25call);
	else
		if (config->ax25path[0] == '\0')
			strcpy(tmp, config->ax25call);
		else
			sprintf(tmp,"%s %s", config->ax25call, config->ax25path);

#ifdef HAVE_NETAX25_AXLIB_H
	addrlen_d = ax25_aton(tmp,&ax25_d);
#else
	addrlen_d = convert_call(tmp,&ax25_d);
#endif
	if (addrlen_d == -1)
	{
		printf("Error: Cannot convert destinationcall - wrong format?\n\n");
      exit(1);
	}


	fd = socket(AF_AX25,SOCK_SEQPACKET,0);
   if (fd < 0)
   {
		printf("Error: Cannot add socket.\n\n");
      exit(1);
	}
  
	ax25.fsa_ax25.sax25_family = AF_AX25;
	ax25.fsa_ax25.sax25_ndigis = 1;
#ifdef HAVE_NETAX25_AXLIB_H
	ax25_aton_entry(config->ax25mycall,
							ax25.fsa_ax25.sax25_call.ax25_call);
	ax25_aton_entry(ax25_config_get_addr(config->ax25port),
							ax25.fsa_digipeater[0].ax25_call);
#else
	convert_call_entry(config->ax25mycall,
							ax25.fsa_ax25.sax25_call.ax25_call);
	convert_call_entry(ax25_config_get_addr(config->ax25port),
							ax25.fsa_digipeater[0].ax25_call);
#endif
	addrlen = sizeof(struct full_sockaddr_ax25);


	if (bind(fd, (struct sockaddr *)&ax25, addrlen) == -1)
	{
   	printf("Error: Cannot bind socket.\n%s\n\n", strerror(errno));
		close(fd);
      exit(1);
	}

	i = TRUE;
	ioctl(fd, FIONBIO, &i);

	if (connect(fd, (struct sockaddr *)&ax25_d, addrlen_d))
	{
		if (errno != EINPROGRESS)
		{
      	printf("Error: Cannot connect to destination. (connect()-error)\n\n");
			close(fd);
         exit(1);
		}
	}
}


void Forward::sendString( char *string, bool save )
{
	sendString( strlen(string), string, save );
}


void Forward::sendString( int len, char *string, bool save )
{
	char *tmp;


   tmp = (char *) malloc(len+sendstrlen);
   if (sendstr != NULL)
   {
	   memcpy(tmp, sendstr, sendstrlen);
      free(sendstr);
	}
	memcpy(tmp+sendstrlen, string, len);
   sendstrlen += len;

   while (sendstrlen >= paclen)
   {
   	write(fd, tmp, paclen);
      sendstrlen -= paclen;
      memmove( tmp, tmp+paclen, sendstrlen );
   }

   if (save)
   {
      sendstr = tmp;
   }
   else
   {
		write(fd, tmp, sendstrlen);
      sendstr = NULL;
      sendstrlen = 0;
      free(tmp);
   }
}


void Forward::sendForwardRequest()
{
	char buffer[500];


   // SEND-Befehl fuer den ersten Eintrag abschicken
   sprintf(buffer, "S %s@%s $%s < %s\r",
   		msgroot->to, msgroot->mbx, msgroot->bid, msgroot->from);
   sendString( buffer );
}


void Forward::nextMsg()
{
	s_msg *tmp;

   tmp = msgroot;
   msgroot = msgroot->next;

   if (tmp->bid != NULL)
   	free(tmp->bid);
   if (tmp->to != NULL)
   	free(tmp->to);
   if (tmp->mbx != NULL)
   	free(tmp->mbx);
   if (tmp->from != NULL)
   	free(tmp->from);
	if (tmp->subject != NULL)
   	free(tmp->subject);
	free(tmp);
}


void Forward::nextForward()
{
	vjboxUnlockBID( msgroot->bid );
	nextMsg();
	if (msgroot == NULL)
	{
   	// S&F-Upload fertig. Download oder Programm beenden.
      if (!config->downloadmail)
      {
      	showStatistik();
	      exit(1);
		}
      status = STATUS_WAITSENDCMD;
      sendString( "F>\r" );
	}
	else
	{
		sendForwardRequest();
		vjboxLockBID( msgroot->bid );
		status = STATUS_GETSENDANSW;
	}
}


void Forward::checkSendCmd( char *line )
{
	char type=' ';
   int len=strlen(line);
   int i;
   char tmp[500];
   char bid[100]="";
   char call[100]="";
   char mbx[100]="";
   char sender[100]="";


	if (line[0] != 'S') return;

   switch (line[1])
   {
   	case 'P':
      case 'B':
      case 'A': type = line[1];
      default: type = '?';
	}

   if ((i = POS(' ', line)) == -1) return;

   len = strlen(line)-i-1;
   memmove(line, line+i+1, len);
   line[len] = '\0';

   if ((i = POS('@', line)) != -1)
   {
   	while (line[i+1] == ' ')
	      delChar( line, i+1 );
		while (line[i-1] == ' ')
	      delChar( line, i-1 );
	}

   if ((i = POS('$', line)) != -1)
   	while (line[i+1] == ' ')
      	delChar( line, i+1 );

   if ((i = POS('<', line)) != -1)
   	while (line[i+1] == ' ')
      	delChar( line, i+1 );


	while (line[0] != '\0')
   {
   	if ((i = POS(' ', line)) == -1)
      {
      	memcpy(tmp, line, len);
         tmp[len] = '\0';
         line[0] = '\0';
      }
      else
      {
      	memcpy( tmp, line, i);
         tmp[i] = '\0';
         len = strlen(line)-i-1;
         memmove(line, line+i+1, len);
         line[len] = '\0';
      }

      switch (tmp[0])
      {
      	case '$': strcpy( bid, tmp+1 ); break;
         case '<': strcpy( sender, tmp+1 ); break;
         default:
         	if ((i = POS('@', tmp)) == -1)
               strcpy(call, tmp);
            else
            {
               memcpy(call, tmp, i);
               call[i] = '\0';
               len = strlen(tmp)-i-1;
               memcpy(mbx, tmp, len);
               mbx[len] = '\0';
            }
      }
   }


   getmsg.call = (char *) strdup(call);
   if (mbx[0] == '\0')
		getmsg.mbx = (char *) strdup("?");
   else
	   getmsg.mbx = (char *) strdup(mbx);
   getmsg.sender = (char *) strdup(sender);
   if (bid[0] == '\0')
		getmsg.bid = (char *) strdup("?");
   else
		getmsg.bid = (char *) strdup(bid);
	getmsg.type = type;
   getmsg.subject = NULL;
   getmsg.lt = 365;

   vjbox->putMsg();
}


void Forward::rejectMail( int grund )
{
	char tmp[500];

	switch (grund)
   {
/*      case REJECT_BIDEXISTS:
      	sprintf( tmp, "NO - BID ALREADY AVAILABLE\r>\r" );
      	break;*/
   	default:
      	sprintf( tmp, "NO\r>\r" );
      	break;
   }

   sendString( tmp );
}


void Forward::getMail()
{
	// Die Mail soll empfangen werden. Erstmal in ein File schreiben.
	sprintf(filefdname, "/tmp/vjf%li.%i", time(NULL), getpid());
	if ((filefd = open(filefdname, O_RDWR|O_CREAT)) == -1)
   {
   	printf("Error: Cannot open temporary file.\n(%s)\n\n", filefdname);
   	exit(1);
   }

	fchmod( filefd, S_IRUSR|S_IWUSR );

   sendString( "OK\r" );

   status = STATUS_GETSUBJECT;
}


bool Forward::checkBINHeader( char *line )
{
	char tmp[500], str[500];
	int i;


   strcpy(tmp, line+5);

   size = -1;
   crc = 0;
   crc_rec = false;

   while (tmp[0] != '\0')
   {
      if ((i = POS('#',tmp)) == -1)
      {
         strcpy(str,tmp);
         tmp[0] = '\0';
      }
      else
      {
         COPY(str,tmp,0,i);
         COPY(tmp,tmp,i+1,strlen(tmp)-i-1);
      }

      switch (str[0])
      {
         case '|': // CRC
                   crc = atoi(str+1);
                   crc_rec = true;
                   break;
        default: if (size == -1)
                    size = atoi(str);
      }
   }

   if (size == -1) return false;

   size_got = 0;
   linemode = false;
   own_crc = 0;

   return true;
}


void Forward::mailSaveToFile( int & len, char *data, char *parent )
{
	if (linemode)
   {
   	if (data[0] == '\032' || !strcmp(data, "/ex") || !strcmp(data, "/EX"))
      {
      	// Ok, die Mail ist fertig
         close(filefd);
         vjbox->saveMail( filefdname );
      	return;
      }
		if (!strncmp(data, "#BIN#", 5))
      	if (checkBINHeader(data))
         {
         }
      write(filefd, data, strlen(data));
      write(filefd, "\n", 1);
	}
   else
   {
		saveBINData( data, len, parent );
   }
}


void Forward::saveBINData( char *data, int & len, char *parent )
{
   if (size-size_got > len)
   {
      write( filefd, data, len );
      size_got += len;
 		own_crc = calc_crcthp_str(data, len, own_crc);
   	return;
   }

   write( filefd, data, size-size_got );
	own_crc = calc_crcthp_str(data, size-size_got, own_crc);

   len -= (size-size_got);
   memcpy( parent, data+(size-size_got), len );
   linemode = true;
   if (len > 0)
   	ready = false;

	close(filefd);

   if (own_crc == crc || !crc_rec)
   {
		vjbox->saveMail( filefdname );
   	return;
   }

	unlink(filefdname);
	printf("Error: CRC-Error (%i != %i)\n\n", own_crc, crc);
	exit(1);
}


void Forward::getMailReady()
{
   sendString( ">\r" );
   status = STATUS_WAITSENDCMD;
}


void Forward::rxFrame()
{
   int i, len;
   char str[5001], str2[5001];
   char buf[5001];
   int count;


   count = read(fd, buf, 5000);
   buf[count] = '\0';


   ready = false;

   len = 0;
   if (savestr != NULL)
   {
      memcpy(str, savestr, savestrlen);
      len = savestrlen;
      free(savestr);
      savestr = NULL;
      savestrlen = 0;
   }

   memcpy(str+len, buf, count);
   len += count;
   str[len] = '\0';


   while (!ready)
   {
   	if (linemode)
      {
	      if ((i = POS('\r', str)) == -1)
	      {
	         // Kein Return - abspeichern
	         savestr = (char *) malloc(len);
	         memcpy(savestr, str, len);
	         savestrlen = len;
   	      return;
	      }
	      else
	      {
	         memcpy( str2, str, i );
	         str2[i] = '\0';
	         len -= (i+1);
	         memmove( str, str+i+1, len );
	         str[len] = '\0';
	      }
		}
      else
      {
      	memcpy(str2, str, len);
         str[0] = '\0';
         ready = true;
      }


if (linemode)
      printf("%s\n", str2);

      switch (status)
      {
      	case STATUS_DEFAULT:
         	if (str2[0] == '>')
            {
            	sendString( MYSID );
               status = STATUS_SENDSID;
				}
         	break;
			case STATUS_SENDSID:
         	if (str2[0] == '>')
            {
            	if (msgroot != NULL)
               {
	               sendForwardRequest();
	               vjboxLockBID( msgroot->bid );
	               status = STATUS_GETSENDANSW;
					}
               else
               {
               	if (!config->downloadmail)
                  {
                     showStatistik();
                     exit(1);
                  }
				      status = STATUS_WAITSENDCMD;
				      sendString( "F>\r" );
               }
				}
            break;
			case STATUS_GETSENDANSW:
            if (!strcmp(str2, "OK"))
            {
            	status = STATUS_WAITSENDMSG;
            	vjbox->sendCurrentMail();
            }
            if (!strcmp(str2, "NO"))
            	nextForward();
         	break;
			case STATUS_WAITSENDMSG:
         	if (str2[0] == '>')
            {
	         	vjboxUnlockBID( msgroot->bid );
	            vjboxAckBID( msgroot->bid );
               nextForward();
				}
         	break;
			case STATUS_WAITSENDCMD:
         	checkSendCmd( str2 );
         	break;
			case STATUS_GETSUBJECT:
            getmsg.subject = (char *) strdup(str2);
            status = STATUS_GETMAIL;
         	break;
			case STATUS_GETMAIL:
            mailSaveToFile( len, str2, str );
         	break;
      }
   }
}





