
#include "vjbox.h"
#include "global.h"
#include "toolbox.h"
#include "main.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#define STATUS_DEFAULT 0
#define STATUS_WAITCONNECT 1
#define STATUS_WAITFWDLIST 2
#define STATUS_WAITHEADER 3
#define STATUS_GETHEADER 4
#define STATUS_GETPOINT 5
#define STATUS_IGNOREHEADER 6
#define STATUS_GETMAIL 7
#define STATUS_CHECKBID 8
#define STATUS_PUTANSWER 9


extern Forward *forward;


int make_connect( unsigned short port, unsigned long int addr )
{
   struct sockaddr_in sock_addr;
   int sock_fd;

   // Socket oeffnen
   if ((sock_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
      return -1;

   // Adressen fuer die Connect-Geschichte festlegen
   sock_addr.sin_family = AF_INET;
   sock_addr.sin_port = htons(port);
   sock_addr.sin_addr.s_addr = addr;

   // Und jetzt connecten
   if (connect(sock_fd, (struct sockaddr *) &sock_addr, sizeof(struct sockaddr)) != 0)
      return -1;


   return sock_fd;
}


/////////////////////////////////////////////////////////////////////////////


VJBox::VJBox()
{
	savestr = NULL;
	savestrlen = 0;
	status = STATUS_DEFAULT;
	linemode = true;

   connectVJBox();
}


VJBox::~VJBox()
{
}


void VJBox::sendString( char *string )
{
	sendString( strlen(string), string );
}


void VJBox::sendString( int len, char *string )
{
	write(fd, string, len );
}


void VJBox::connectVJBox()
{
	if ((fd = make_connect(config->vjboxport, inet_addr(config->vjboxip))) == -1)
   {
   	printf("Cannot reach vjboxd.\n\n");
      exit(1);
   }

}


int VJBox::getStatus( char *str )
{
   char tmp[100];
   int i;

   if ((i = POS(' ', str)) > -1)
      if (i > 2)
      {
         memcpy(tmp, str, i+1);
         tmp[i] = '\0';
         return atoi(tmp);
      }
   return -1;
}


void VJBox::addMsg( char *bid )
{
	s_msg *tmp;

   if (msgroot == NULL)
   {
      // Erster Eintrag
      msgroot = (s_msg *) malloc(sizeof(s_msg));
      tmp = msgroot;
   }
   else
   {
  	   tmp = msgroot;
      while (tmp->next != NULL) tmp = tmp->next;

      tmp->next = (s_msg *) malloc(sizeof(s_msg));
      tmp = tmp->next;
   }

   tmp->next = NULL;
   tmp->bid = (char *) strdup(bid);
   tmp->to = NULL;
   tmp->mbx = NULL;
   tmp->from = NULL;
   tmp->subject = NULL;
}


void VJBox::saveCurrentHeader( s_msg *msg, char *line )
{
	int i, len;
   char tmp[500], tmp2[500];

   if ((i = POS(':', line)) == -1) return;

   memcpy( tmp, line, i );
   tmp[i] = '\0';
   len = strlen(line)-i-1;
   memmove( line, line+i+1, len );
   line[len] = '\0';

   KillSpacesLeft( line );

   if (!strcmp(tmp, "To"))
   {
   	if ((i = POS('@', line)) == -1)
      {
      	Gross(line);
         msg->to = (char *) strdup(line);
         msg->mbx = NULL;
      }
      else
      {
		   memcpy( tmp2, line, i );
		   tmp2[i] = '\0';
		   len = strlen(line)-i-1;
		   memmove( line, line+i+1, len );
		   line[len] = '\0';

         Gross(tmp2);
         msg->to = (char *) strdup(tmp2);
         Gross(line);
         msg->mbx = (char *) strdup(line);
      }
   }

   if (!strcmp(tmp, "From"))
   {
   	Gross(line);
      msg->from = (char *) strdup(line);
	}

	if (!strcmp(tmp, "Subject"))
   	msg->subject = (char *) strdup(line);
}


void VJBox::rxFrame()
{
   int i, len;
   char str[5001], str2[5001];
   char buf[5001];
   int count;


   count = read(fd, buf, 5000);
   buf[count] = '\0';


   len = 0;
   if (savestr != NULL)
   {
      memcpy(str, savestr, savestrlen);
      len = savestrlen;
      free(savestr);
      savestr = NULL;
      savestrlen = 0;
   }

   memcpy(str+len, buf, count);
   len += count;
   str[len] = '\0';


   while (str[0] != '\0')
   {
   	if (linemode)
      {
	      if ((i = POS('\n', str)) == -1)
	      {
	         // Kein Return - abspeichern
	         savestr = (char *) malloc(len);
	         memcpy(savestr, str, len);
	         savestrlen = len;
	         return;
	      }
	      else
	      {
	         memcpy( str2, str, i );
	         str2[i] = '\0';
	         len -= (i+1);
	         memmove( str, str+i+1, len );
	         str[len] = '\0';
	      }
		}
		else
      {
      	memcpy(str2, str, len);
         str[0] = '\0';
      }

      printf("%s\n", str2);

      switch (status)
      {
      	case STATUS_WAITCONNECT:
         	if (str2[0] == '.')
            {
            	sendString( "fwdlist\n" );
					status = STATUS_WAITFWDLIST;
				}
         	break;
			case STATUS_WAITFWDLIST:
         	if (str2[0] == '.')
            {
               currmsg = msgroot;
               if (currmsg != NULL)
               {
	               status = STATUS_GETHEADER;
                  sprintf(buf, "header %s\n", currmsg->bid);
                  sendString( buf );
					}
					else
               {
               	if (!config->downloadmail)
                  {
                  	printf("No mail available for uploading.\n\n");
                  	exit(1);
                  }
		         	status = STATUS_DEFAULT;
						if (forward == NULL)
	                  forwardConnect();
					}
            }
            else
            	addMsg( str2 );
         	break;
			case STATUS_GETHEADER:
				if (str2[0] == '.')
            {
            	currmsg = currmsg->next;
               if (currmsg == NULL)
               {
		         	status = STATUS_DEFAULT;
						if (forward == NULL)
	                  forwardConnect();
               }
               else
               {
	               status = STATUS_GETHEADER;
                  sprintf(buf, "header %s\n", currmsg->bid);
                  sendString( buf );
               }
            }
            else
            	saveCurrentHeader( currmsg, str2 );
         	break;
			case STATUS_GETPOINT:
         	if (str2[0] == '.')
            	status = STATUS_DEFAULT;
         	break;
 			case STATUS_IGNOREHEADER:
         	if (str2[0] == '\0')
            {
            	status = STATUS_GETMAIL;
//               linemode = false;
				}
         	break;
			case STATUS_GETMAIL:
            if (str2[0] == '.')
            {
            	linemode = true;
               status = STATUS_DEFAULT;
               sprintf( buf, "%c\r", 26 );
               forward->sendString( 2, buf );
				}
            else
               msgData(str2, len);
            break;
			case STATUS_PUTANSWER:
            switch (getStatus(str2))
            {
            	case 162:
            		sprintf(buf, "%s\n", getmsg.subject);
            		sendString( buf );
                  break;
					case 164:
						sendMailToVjbox();
               	break;
					case 165:
               	printf("Error: Access to the vjbox-board '%s' denied.\n\n", getmsg.call);
                  exit(1);
               	break;
               case 167:
               	printf("Error: Unknown destination box.\n\n");
                  exit(1);
                  break;
					case 168:
                  printf("Error: Wrong filetype.\n\n");
                  exit(1);
               	break;
               case 169:
               	printf("Error: BID is already known.\n\n");
                  exit(1);
               	break;
					case 166:
               	status = STATUS_DEFAULT;
                  forward->getMailReady();
               	break;
            }
         	break;
      	default:
		      switch (getStatus(str2))
		      {
		      	case 200:
			     		sprintf(buf, "login %s\n", config->vjboxname);
			     		sendString( buf );
                  status = STATUS_WAITCONNECT;
						break;
               case 401:
                  forward->rejectMail( REJECT_BIDEXISTS );
               	break;
					case 402:
						forward->getMail();
               	break;
		      }
      }
   }
}


void VJBox::lockBID( const char *bid )
{
	char buf[500];


   sprintf( buf, "fwdlock %s\n", bid );
   status = STATUS_GETPOINT;
   sendString( buf );
}


void VJBox::unlockBID( const char *bid )
{
	char buf[500];


   sprintf( buf, "fwdunlock %s\n", bid );
   status = STATUS_GETPOINT;
   sendString( buf );
}


void VJBox::ackBID( const char *bid )
{
	char buf[500];


   sprintf( buf, "fwdack %s\n", bid );
   status = STATUS_GETPOINT;
   sendString( buf );
}


void VJBox::sendCurrentMail()
{
	char buf[500];


   // Subject senden
   sprintf(buf, "%s\r", msgroot->subject);
	forward->sendString( buf );

   sprintf( buf, "getmail %s\n", msgroot->bid );
   sendString( buf );
   status = STATUS_IGNOREHEADER;
}


void VJBox::msgData( char *data, int len )
{
	if (linemode)
   {
   	if (!strncmp(data, "#BIN#", 5))
      {
      }
      forward->sendString( data, true );
      forward->sendString( 1, "\r", true );
   }
   else
   {
	}
}


void VJBox::putMsg()
{
	char tmp[500];


   if (getmsg.bid[0] == '?')
   {
	   forward->getMail();
      return;
	}

   sprintf(tmp, "BIDEXIST %s\n", getmsg.bid);
   sendString( tmp );
   status = STATUS_DEFAULT;
}


void VJBox::saveMail( char *filename )
{
	char tmp[500];
   struct stat st;


   stat(filename, &st);

	sprintf(tmp, "PUT %s %s %c %s %s %i %li\n",
   		getmsg.sender, getmsg.bid, getmsg.type, getmsg.call,
         getmsg.mbx, getmsg.lt, st.st_size);

	sendString( tmp );
   status = STATUS_PUTANSWER;
   strcpy(sendfname, filename);
}


void VJBox::sendMailToVjbox()
{
	int fd;
	int count;
	char tmp[5000];


	if ((fd = open(sendfname, O_RDONLY)) == -1)
	{
		printf("Error: Cannot open temporary file\n(%s)\n\n", sendfname);
		exit(1);
	}

	do
	{
		count = read(fd, tmp, 5000);
		sendString( count, tmp );
	} while (count == 5000);

	close(fd);
   unlink(sendfname);
}



